import inspect
from datetime import datetime, date, time
from decimal import Decimal
from uuid import UUID
from typing import Any, Set, Dict

from sthg_common_base.utils.constants import Constants


class EnhancedJSONSerializer:
    """通用对象到字符串转换器"""

    # 需要排除的模块前缀（如框架内部对象）
    EXCLUDED_MODULES = {
        'sqlalchemy.', 'starlette.', 'fastapi.',
        'requests.', 'socket.', '_io.', 'builtins.'
    }

    # 最大递归深度
    MAX_DEPTH = 2

    # 类型特殊处理白名单
    SPECIAL_HANDLERS = {
        datetime: lambda o: o.isoformat(),
        date: lambda o: o.isoformat(),
        time: lambda o: o.isoformat(),
        Decimal: lambda o: f"Decimal({str(o)})",
        UUID: lambda o: str(o),
        complex: lambda o: f"{o.real}+{o.imag}j",
        bytes: lambda o: f"b'{o.hex()}'"
    }

    @classmethod
    def register_excluded_modules(cls, *modules: str) -> None:
        """注册需要排除的模块前缀到集合

        Args:
            modules: 可变数量的字符串参数，例如 'pymysql.', 'django.'
        """
        for module in modules:
            if isinstance(module, str):
                cls.EXCLUDED_MODULES.add(module)

    @classmethod
    def json_serializer(cls, obj: Any) -> str:
        """主转换入口"""
        return cls._convert(obj, depth=0, memo=set())

    @classmethod
    def _convert(cls, obj: Any, depth: int, memo: Set[int]) -> str:
        """递归转换核心方法"""
        try:
            # 基本类型快速返回
            if obj is None:
                return "null"
            if isinstance(obj, (int, float, bool)):
                return str(obj)
            if isinstance(obj, str):
                return f'"{obj}"'  # 保留字符串边界

            # 检测循环引用
            obj_id = id(obj)
            if obj_id in memo:
                return "<Circular Reference>"
            memo.add(obj_id)

            # 深度限制
            if depth >= cls.MAX_DEPTH:
                memo.remove(obj_id)
                return "<Max Depth Reached>"

            # 特殊类型处理
            for typ, handler in cls.SPECIAL_HANDLERS.items():
                if isinstance(obj, typ):
                    return handler(obj)

            # 排除危险类型
            if cls._is_excluded(obj):
                return f"<{type(obj).__name__} excluded>"

            # 容器类型处理
            if isinstance(obj, (list, tuple, set)):
                return cls._convert_iterable(obj, depth, memo)
            if isinstance(obj, dict):
                return cls._convert_dict(obj, depth, memo)
            # 自定义对象处理
            return cls._convert_custom(obj, depth, memo)
        except Exception as ex:
            return f"<Conversion Error: {Constants.Str_Place}>"

    @classmethod
    def _convert_iterable(cls, iterable, depth: int, memo: Set[int]) -> str:
        """处理可迭代对象"""
        items = [
            cls._convert(item, depth + 1, memo)
            for item in iterable
        ]
        if isinstance(iterable, tuple):
            return f"({', '.join(items)})"
        if isinstance(iterable, set):
            return f"{{{', '.join(items)}}}"
        return f"[{', '.join(items)}]"

    @classmethod
    def _convert_dict(cls, dct: Dict, depth: int, memo: Set[int]) -> str:
        """处理字典"""
        pairs = []
        for k, v in dct.items():
            key_str = cls._convert(k, depth + 1, memo)
            val_str = cls._convert(v, depth + 1, memo)
            pairs.append(f"{key_str}: {val_str}")
        return f"{{{', '.join(pairs)}}}"

    @classmethod
    def _convert_custom(cls, obj: Any, depth: int, memo: Set[int]) -> str:
        """处理自定义对象"""
        try:
            # 尝试获取字典表示
            if hasattr(obj, '__dict__'):
                content = cls._convert(vars(obj), depth + 1, memo)
                return f"{type(obj).__name__}({content})"

            # 处理其他可调用对象
            if callable(obj):
                return f"<callable {type(obj).__name__}>"

            # 默认字符串表示
            return f'"{str(obj)}"'
        except Exception as e:
            return f"<Conversion Error: {Constants.Str_Place}>"
        finally:
            memo.discard(id(obj))

    @classmethod
    def _is_excluded(cls, obj: Any) -> bool:
        """检查是否需要排除"""
        try:
            module = inspect.getmodule(obj).__name__
            return any(module.startswith(p) for p in cls.EXCLUDED_MODULES)
        except (AttributeError, TypeError):
            return False



if __name__ == '__main__':
    EnhancedJSONSerializer.register_excluded_modules("pymysql.", "django.")



import re

class HttpStatus():
    """HTTP 状态码枚举"""
    CONTINUE = 100
    SWITCHING_PROTOCOLS = 101
    OK = 200
    CREATED = 201
    NO_CONTENT = 204
    Range = 206

    # 重定向状态码
    MOVED_PERMANENTLY = 301
    FOUND = 302
    NOT_MODIFIED = 304

    # 客户端错误状态码
    BAD_REQUEST = 400
    UNAUTHORIZED = 401
    FORBIDDEN = 403
    NOT_FOUND = 404
    CONFLICT = 409
    METHOD_NOT_ALLOWED = 405
    PAYLOAD_TOO_LARGE = 413
    TOO_MANY_REQUESTS = 429
    UNAVAILABLE_FOR_LEGAL_REASONS = 451

    # 服务器错误状态码
    INTERNAL_SERVER_ERROR = 500
    BAD_GATEWAY = 502
    SERVICE_UNAVAILABLE = 503
    GATEWAY_TIMEOUT = 504
    CONNECTION_TIMEOUT = 504
    READ_TIMEOUT = 504

def validate_busi_code(busi_code):
    """
    校验 busiCode 是否符合以下格式之一：
    - DDL_Query_Error
    - DDLQueryError
    - File_Is_NotExist
    要求：
    - 首字母必须大写
    - 单词之间可以用下划线连接，也可以直接驼峰连接
    - 下划线两边必须是大写字母开头的单词
    """
    # 正则：允许多组 (可选下划线 + 大写字母开头+小写字母/数字)
    pattern = re.compile(r"^([A-Z][a-z0-9]*)(_?[A-Z][a-z0-9]*)*$")

    if not pattern.fullmatch(busi_code):
        raise ValueError(
            f"busiCode '{busi_code}' 格式非法，必须符合 Bucket_Not_Exist, BucketNotExist, File_Is_NotExist, DDL_Query_Error 这样的格式。"
        )

class ResCode:
    """
    TempClass represents a structured response entry with:
    - business code (busiCode)
    - business message (busiMsg)
    - HTTP status code (httpCode)
    """
    def __init__(self, busiCode, busiMsg, httpCode):
        # Validate busiCode format using regex
        if not re.match(r'^[A-Z_]+$', busiCode):
            validate_busi_code(busiCode)
        self._busiCode = busiCode
        self._busiMsg = busiMsg
        self._httpCode = httpCode

    @property
    def getBusiCode(self):
        """返回业务编码."""
        return self._busiCode

    @property
    def getBusiMsg(self):
        """返回业务描述."""
        return self._busiMsg

    @property
    def getHttpCode(self):
        """返回HTTP状态码."""
        return self._httpCode


class ResponseEnum:
    """响应枚举类，包含错误代码、描述和 HTTP 状态码"""
    """FileSystem"""
    SpaceNameAlreadyExists = ResCode(
        "CONFLICT",
        "提供的空间名称已存在",
        HttpStatus.CONFLICT
    )
    FolderNotTrashed = ResCode(
        "INVALID_ARGUMENT",
        "文件夹应先直接移入回收站，再进行永久删除",
        HttpStatus.BAD_REQUEST
    )

    # 2. 项目管理相关
    ProjectCreationNotSupported = ResCode(
        "INVALID_ARGUMENT",
        "当前用户空间不支持创建项目",
        HttpStatus.BAD_REQUEST
    )
    ProjectNameAlreadyExists = ResCode(
        "CONFLICT",
        "所请求的项目显示名称在空间中已被使用",
        HttpStatus.CONFLICT
    )
    CreateProjectNoOwnerLikeRoleGrant = ResCode(
        "INVALID_ARGUMENT",
        "创建项目请求未授予任何主体类所有者角色，导致项目无管理员权限。若角色包含'compass:edit-project'操作则定义为类所有者角色，默认角色集中通常为'compass:manage'角色",
        HttpStatus.BAD_REQUEST
    )
    CreateProjectPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法创建项目",
        HttpStatus.FORBIDDEN
    )
    ProjectNotFound = ResCode(
        "NOT_FOUND",
        "未找到指定项目",
        HttpStatus.NOT_FOUND
    )

    # 3. 空间管理相关
    SpaceNotFound = ResCode(
        "NOT_FOUND",
        "未找到指定空间",
        HttpStatus.NOT_FOUND
    )
    GetSpaceResourceNotSupported = ResCode(
        "INVALID_ARGUMENT",
        "不支持将空间作为资源获取",
        HttpStatus.BAD_REQUEST
    )

    # 4. 组织管理相关
    OrganizationsNotFound = ResCode(
        "NOT_FOUND",
        "至少一个组织RID未找到",
        HttpStatus.NOT_FOUND
    )
    OrganizationMarkingNotOnSpace = ResCode(
        "INVALID_ARGUMENT",
        "传入组织的至少一个组织标记未应用于请求的空间",
        HttpStatus.BAD_REQUEST
    )
    OrganizationCannotBeRemoved = ResCode(
        "INVALID_ARGUMENT",
        "若移除组织会导致标记了组织的空间下的项目无组织，则无法从项目中移除该组织",
        HttpStatus.BAD_REQUEST
    )
    InvalidOrganizationHierarchy = ResCode(
        "INVALID_ARGUMENT",
        "项目中的组织必须也存在于父空间中。若项目组织配置（创建时或后续）导致标记空间中无组织，或包含父空间不存在的组织，则抛出此错误",
        HttpStatus.BAD_REQUEST
    )
    RemoveOrganizationsPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法从项目中移除组织",
        HttpStatus.FORBIDDEN
    )
    AddOrganizationsPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法向项目中添加组织",
        HttpStatus.FORBIDDEN
    )

    # 5. 角色权限相关
    InvalidRoleIds = ResCode(
        "INVALID_ARGUMENT",
        "默认角色或角色授予中引用的roleId在空间的项目角色集中不存在",
        HttpStatus.BAD_REQUEST
    )

    # 6. 文件夹与资源管理相关
    InvalidFolder = ResCode(
        "INVALID_ARGUMENT",
        "给定资源不是文件夹",
        HttpStatus.BAD_REQUEST
    )
    GetRootFolderNotSupported = ResCode(
        "INVALID_ARGUMENT",
        "不支持将根文件夹作为资源获取",
        HttpStatus.BAD_REQUEST
    )
    FolderNotFound = ResCode(
        "NOT_FOUND",
        "未找到指定文件夹",
        HttpStatus.NOT_FOUND
    )
    ResourceNotFound = ResCode(
        "NOT_FOUND",
        "未找到指定资源",
        HttpStatus.NOT_FOUND
    )
    ResourceNameAlreadyExists = ResCode(
        "CONFLICT",
        "提供的资源名称在同一文件夹中已被其他资源使用",
        HttpStatus.CONFLICT
    )
    CreateFolderOutsideProjectNotSupported = ResCode(
        "INVALID_ARGUMENT",
        "不支持在项目外部创建文件夹",
        HttpStatus.BAD_REQUEST
    )
    CreateFolderPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法创建文件夹",
        HttpStatus.FORBIDDEN
    )

    # 7. 显示名称相关
    InvalidDisplayName = ResCode(
        "INVALID_ARGUMENT",
        "资源的显示名称不能是'.'或'..'、包含正斜杠'/'或过长",
        HttpStatus.BAD_REQUEST
    )
    MissingDisplayName = ResCode(
        "INVALID_ARGUMENT",
        "必须提供显示名称",
        HttpStatus.BAD_REQUEST
    )

    # 8. 访问需求相关
    GetAccessRequirementsPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法获取资源的访问需求",
        HttpStatus.FORBIDDEN
    )

    # 9. 组织标记相关
    OrganizationMarkingNotSupported = ResCode(
        "INVALID_ARGUMENT",
        "不支持将组织标记作为常规标记添加。请改用项目资源上的组织端点",
        HttpStatus.BAD_REQUEST
    )
    MarkingNotFound = ResCode(
        "NOT_FOUND",
        "提供的标记ID无法找到",
        HttpStatus.NOT_FOUND
    )
    AddMarkingsPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法向资源添加标记",
        HttpStatus.FORBIDDEN
    )
    RemoveMarkingsPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法从资源移除标记",
        HttpStatus.FORBIDDEN
    )

    # 10. 资源操作限制相关
    ForbiddenOperationOnHiddenResource = ResCode(
        "INVALID_ARGUMENT",
        "不支持对隐藏资源执行此操作",
        HttpStatus.BAD_REQUEST
    )
    ForbiddenOperationOnAutosavedResource = ResCode(
        "INVALID_ARGUMENT",
        "不支持对自动保存的资源执行此操作",
        HttpStatus.BAD_REQUEST
    )

    # 11. 资源删除与恢复相关
    ResourceNotTrashed = ResCode(
        "INVALID_ARGUMENT",
        "资源应先直接移入回收站，再进行永久删除",
        HttpStatus.BAD_REQUEST
    )
    PermanentlyDeleteResourcePermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法永久删除资源",
        HttpStatus.FORBIDDEN
    )
    ResourceNotDirectlyTrashed = ResCode(
        "INVALID_ARGUMENT",
        "资源未直接移入回收站",
        HttpStatus.BAD_REQUEST
    )
    RestoreResourcePermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法恢复资源",
        HttpStatus.FORBIDDEN
    )

    # 12. 路径相关
    PathNotFound = ResCode(
        "NOT_FOUND",
        "未找到指定路径",
        HttpStatus.NOT_FOUND
    )
    InvalidPath = ResCode(
        "INVALID_ARGUMENT",
        "指定路径无效。有效路径的所有组件均由单个'/'分隔",
        HttpStatus.BAD_REQUEST
    )
    GetByPathPermissionDenied = ResCode(
        "PERMISSION_DENIED",
        "无法通过路径获取资源",
        HttpStatus.FORBIDDEN
    )

    AccessDenied = ResCode("AccessDenied", "访问被拒绝", HttpStatus.FORBIDDEN)
    InternalError = ResCode("InternalError", "我们遇到了内部错误。请稍后再试。", HttpStatus.INTERNAL_SERVER_ERROR)
    InvalidArgument = ResCode("InvalidArgument", "无效参数", HttpStatus.BAD_REQUEST)
    InvalidRequest = ResCode("InvalidRequest", "SOAP 请求必须通过 HTTPS 连接发送。", HttpStatus.BAD_REQUEST)
    InvalidURI = ResCode("InvalidUri", "无法解析指定的 URI。", HttpStatus.BAD_REQUEST)
    RequestTimeout = ResCode("RequestTimeout", "您的套接字连接在超时期间内未读取或写入数据。", HttpStatus.GATEWAY_TIMEOUT)
    ReadTimeout = ResCode("ReadTimeout", "数据读取超时", HttpStatus.READ_TIMEOUT)
    ConnectionTimeout = ResCode("ConnectionTimeout", "请求连接超时", HttpStatus.CONNECTION_TIMEOUT)
    ConnectionError = ResCode("ConnectionError", "请求错误!", HttpStatus.BAD_GATEWAY)
    NotFoundApi = ResCode("NotFoundApi", "未找到 API", HttpStatus.NOT_FOUND)
    NotFoundData = ResCode("NotFoundData", "未找到数据", HttpStatus.NOT_FOUND)
    URLExpired = ResCode("UrlExpired", "URL 已过期，请获取一个新的。", HttpStatus.FORBIDDEN)
    InvalidKey = ResCode("InvalidKey", "无效密钥", HttpStatus.BAD_REQUEST)
    Unauthorized = ResCode("Unauthorized", "请求需要用户身份验证", HttpStatus.UNAUTHORIZED)
    NotifyFail = ResCode("NotifyFail", "通知失败", HttpStatus.BAD_REQUEST)
    CallbackFail = ResCode("CallbackFail", "回调服务器失败", HttpStatus.BAD_REQUEST)
    OK = ResCode("OK", "成功", HttpStatus.OK)
    Repeat_Request = ResCode("Repeat_Request", "重复请求", HttpStatus.OK)
    Sign_Is_Not_Pass = ResCode("Sign_Is_Not_Pass", "签名未通过", HttpStatus.BAD_REQUEST)
    Error_Token = ResCode("Error_Token", "非法授权", HttpStatus.UNAUTHORIZED)
    Payload_Too_Large = ResCode("Payload_Too_Large", "负载过大！", HttpStatus.FORBIDDEN)
    Too_Many_Request = ResCode("Too_Many_Request", "您已被限制，请稍后再试！", HttpStatus.TOO_MANY_REQUESTS)
    Meta_Data_Error = ResCode("Meta_Data_Error", "元数据错误！", HttpStatus.BAD_REQUEST)
    Param_Error = ResCode("Param_Error", "参数错误", HttpStatus.BAD_REQUEST)
    Time_Error = ResCode("Time_Error", "您的时间参数不正确或已过期！", HttpStatus.BAD_REQUEST)
    Rule_Not_Found = ResCode("Rule_Not_Found", "规则未找到！", HttpStatus.NOT_FOUND)
    Service_Timeout = ResCode("Service_Timeout", "服务调用超时！", HttpStatus.GATEWAY_TIMEOUT)
    Sign_Time_Is_Timeout = ResCode("Sign_Time_Is_Timeout", "签名时间戳已超过 %s 分钟！", HttpStatus.GATEWAY_TIMEOUT)
    Sentinel_Block_Error = ResCode("Sentinel_Block_Error", "请求被 Sentinel 阻止！", HttpStatus.BAD_REQUEST)
    Decryption_Error = ResCode(
        "Decryption_Error", "解密失败，请检查参数或密钥，或数据长度过长", HttpStatus.UNAVAILABLE_FOR_LEGAL_REASONS)
    Ecryption_Error = ResCode(
        "Ecryption_Error", "加密失败，请检查参数或密钥，或数据长度过长", HttpStatus.UNAVAILABLE_FOR_LEGAL_REASONS)
    Invalid_Xml_Data = ResCode("Invalid_Xml_Data", "XML 数据无效。", HttpStatus.BAD_REQUEST)
    Request_Header_Too_Large = ResCode("Request_Header_Too_Large", "请求头字段过大", HttpStatus.PAYLOAD_TOO_LARGE)
    Request_Entity_Too_Large = ResCode("Request_Entity_Too_Large", "请求实体过大", HttpStatus.PAYLOAD_TOO_LARGE)
    Security_Forbidden = ResCode("Security_Forbidden", "安全禁止", HttpStatus.FORBIDDEN)
    Redis_Error = ResCode("Redis_Error", "Redis 错误", HttpStatus.INTERNAL_SERVER_ERROR)
    Mysql_Error = ResCode("Mysql_Error", "MySQL 错误", HttpStatus.INTERNAL_SERVER_ERROR)
    Request_Header_Invalid = ResCode("Request_Header_Invalid", "请求头错误", HttpStatus.BAD_REQUEST)
    Source_Switching_Failed = ResCode("Source_Switching_Failed", "数据源切换保存", HttpStatus.INTERNAL_SERVER_ERROR)
    Third_Service_Error = ResCode("Third_Service_Error", "第三方服务错误", HttpStatus.INTERNAL_SERVER_ERROR)
    Data_Write_Failed = ResCode("Data_Write_Failed", "数据写入磁盘失败。", HttpStatus.BAD_REQUEST)
    File_Drop_Field = ResCode("File_Drop_Field", "文件丢弃字段", HttpStatus.BAD_REQUEST)
    File_Create_Field = ResCode("File_Create_Field", "文件创建字段", HttpStatus.BAD_REQUEST)
    PGsql_Error = ResCode("PGsql_Error", "PostgreSQL 错误", HttpStatus.BAD_REQUEST)
    PGsql_Insert_Error = ResCode("PGsql_Insert_Error", "PostgreSQL 插入错误", HttpStatus.BAD_REQUEST)
    PGsql_Update_Error = ResCode("PGsql_Update_Error", "PostgreSQL 更新错误", HttpStatus.BAD_REQUEST)
    PGsql_Query_Error = ResCode("PGsql_Query_Error", "PostgreSQL 查询错误", HttpStatus.BAD_REQUEST)
    Data_Not_Complete = ResCode("Data_Not_Complete", "数据不完整！", HttpStatus.BAD_REQUEST)
    Too_Much_Data = ResCode("Too_Much_Data", "数据过多，停止导出", HttpStatus.BAD_REQUEST)
    Table_No_UniqueIndex = ResCode("Table_No_UniqueIndex", "", HttpStatus.BAD_REQUEST)
    Table_Parse_Failed = ResCode("Table_Parse_Failed", "表解析失败", HttpStatus.BAD_REQUEST)
    Still_Failed_After_Retry = ResCode("Still_Failed_After_Retry", "重试后仍然失败", HttpStatus.BAD_REQUEST)
    Zero_Export_Data = ResCode("Zero_Export_Data", "零导出数据", HttpStatus.OK)
    Zero_Import_Data = ResCode("Zero_Import_Data", "零导入数据", HttpStatus.OK)
    Database_Is_Empty = ResCode("Database_Is_Empty", "数据库中的表为空", HttpStatus.OK)
    Folder_Is_Empty = ResCode("Folder_Is_Empty", "文件夹为空", HttpStatus.OK)
    File_Is_NotExist = ResCode("File_Is_NotExist", "文件不存在", HttpStatus.OK)
    Bucket_Not_Exist = ResCode("Bucket_Not_Exist", "存储桶不存在", HttpStatus.BAD_REQUEST)
    Final_Faild = ResCode("Final_Faild", "任务失败", HttpStatus.BAD_REQUEST)
    DDL_Query_Error = ResCode("DDL_Query_Error", "查询 DDL 错误", HttpStatus.BAD_REQUEST)
    JSON_Transform_Error = ResCode("JSON_Transform_Error", "JSON 转换错误", HttpStatus.BAD_REQUEST)
    Directory_Create_Error = ResCode("Directory_Create_Error", "目录创建错误", HttpStatus.BAD_REQUEST)
    File_Write_Error = ResCode("File_Write_Error", "文件写入错误", HttpStatus.BAD_REQUEST)
    Directory_Not_Exist = ResCode("Directory_Not_Exist", "目录不存在", HttpStatus.BAD_REQUEST)
    File_Read_Error = ResCode("File_Read_Error", "文件读取错误", HttpStatus.BAD_REQUEST)
    Sequence_Query_Error = ResCode("Sequence_Query_Error", "序列查询错误", HttpStatus.BAD_REQUEST)
    Sequence_Create_Error = ResCode("Sequence_Create_Error", "序列创建错误", HttpStatus.BAD_REQUEST)
    Export_File_Error = ResCode("Export_File_Error", "导出文件错误", HttpStatus.BAD_REQUEST)
    Import_Xml_Error = ResCode("Import_Xml_Error", "导入 XML 错误", HttpStatus.BAD_REQUEST)
    Field_Parsing_Error = ResCode("Field_Parsing_Error", "字段解析错误", HttpStatus.BAD_REQUEST)
    Dir_TARGZ_Error = ResCode("Dir_TARGZ_Error", "目录 TAR.GZ 错误", HttpStatus.BAD_REQUEST)
    Minio_Upload_Failed = ResCode("Minio_Upload_Failed", "Minio 上传失败", HttpStatus.BAD_REQUEST)
    Rollback_DDL_Failed = ResCode("Rollback_DDL_Failed", "回滚 DDL 失败", HttpStatus.BAD_REQUEST)
    Rollback_DML_Failed = ResCode("Rollback_DML_Failed", "回滚 DML 失败", HttpStatus.BAD_REQUEST)
    Some_File_Upload_Failed = ResCode("Some_File_Upload_Failed", "部分文件上传失败", HttpStatus.OK)
    Invalid_Tar_Data = ResCode("Invalid_Tar_Data", "tar 名称无效。", HttpStatus.BAD_REQUEST)
    Too_Retry_Request = ResCode("Too_Retry_Request", "您已被限制，请稍后再试！", HttpStatus.BAD_REQUEST)
    DB_Timeout = ResCode("Db_Timeout", "DB服务调用超时！", HttpStatus.GATEWAY_TIMEOUT)
    Redis_Error_Timeout = ResCode("Redis_Error_Timeout", "Redis服务调用超时！", HttpStatus.GATEWAY_TIMEOUT)
    Too_Many_Redirects = ResCode("Too_Many_Redirects", "请求跳转次数太多!", HttpStatus.BAD_REQUEST)
    Invalid_Header = ResCode("Invalid_Header", "请求头异常!", HttpStatus.BAD_REQUEST)
    Invalid_Proxy_URL = ResCode("Invalid_Proxy_URL", "错误的代理地址!", HttpStatus.BAD_REQUEST)
    Job_Execute_Field = ResCode("Job_Execute_Field", "job执行失败!", HttpStatus.BAD_REQUEST)
    Client_Connect_Timeout = ResCode("Client_Connect_Timeout", "数据库连接超时!", HttpStatus.GATEWAY_TIMEOUT)
    Db_Error = ResCode("Db_Error", "数据库错误!", HttpStatus.BAD_REQUEST)

    @classmethod
    def update_response_enum(cls, enum_class):
        """
        动态添加枚举类中的所有 ResCode 实例到 ResponseEnum 中。
        :param enum_class: 包含 ResCode 实例的类
        :raises ValueError: 如果响应项名称已存在
        """
        for name in dir(enum_class):
            if name.startswith('__'):
                continue  # 跳过内置属性
            attr = getattr(enum_class, name)
            if isinstance(attr, ResCode):
                if not hasattr(cls, name):
                    if hasattr(cls, name):
                        raise ValueError(f"ResponseEnum 中已存在 '{name}'，无法重复添加。")
                    setattr(cls, name, attr)

    @classmethod
    @classmethod
    def from_code(cls, code):
        """
        根据 busiCode 获取对应的 ResCode 实例。
        """
        for name in dir(cls):
            if name.startswith('__'):
                continue  # 跳过内置属性
            attr = getattr(cls, name)
            if isinstance(attr, ResCode) and attr.getBusiCode == code:
                return attr
        return None

class UserErrorEnmu():
    GitError = ResCode("GitError", "git产生错误！！", HttpStatus.GATEWAY_TIMEOUT)


if __name__ == '__main__':

    ResponseEnum.update_response_enum(UserErrorEnmu)
    print(ResponseEnum.OK.getBusiCode)
    print(ResponseEnum.GitError.getBusiCode)
    print(ResponseEnum.OK.getBusiMsg)
    print(ResponseEnum.OK.getHttpCode)
    # 通过错误代码获取枚举项
    code = "AccessDenied"
    enum_member = ResponseEnum.from_code(code)
    if enum_member:
        print(enum_member.getBusiMsg)  # 输出: 您已被限制，请稍后再试！



"""
@Author  ：duomei
@File    ：request_util.py
@Time    ：2025/4/28 16:07
"""
import json
from contextvars import ContextVar

import requests
from requests.exceptions import ConnectionError, ConnectTimeout, ReadTimeout, TooManyRedirects, URLRequired, InvalidURL, \
    InvalidHeader, InvalidProxyURL
from starlette.datastructures import Headers, MutableHeaders

from sthg_common_base import BaseResponse, ResponseEnum, format_exception_msg, access_log, LoggerUtil
from sthg_common_base.utils.sthg_common_constants import SthgCommonConstants


class RequestUtil:

    @staticmethod
    @access_log()
    def get(url: str, headers=None, params=None, encoding=None, timeout=30):
        """
        同步get
        :param url: 请求的URL
        :param headers: 请求头
        :param timeout: 超时时间，默认为10秒
        :param params: URL参数
        :param encoding: 编码方式
        :return: 解析后的JSON响应
        """
        try:
            new_headers=RequestUtil.builder_headers(headers)
            response = requests.get(url, headers=new_headers, params=params, timeout=timeout)
            if encoding:
                response.encoding = encoding
            text = response.text
            return BaseResponse(data=json.loads(text), resEnum=ResponseEnum.OK, msg=f"请地址url:{url}")
        except ReadTimeout as e:
            return BaseResponse(resEnum=ResponseEnum.RequestTimeout, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except TooManyRedirects as e:
            return BaseResponse(resEnum=ResponseEnum.Too_Many_Redirects, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except URLRequired as e:
            return BaseResponse(resEnum=ResponseEnum.InvalidRequest, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except InvalidURL as e:
            return BaseResponse(resEnum=ResponseEnum.InvalidURI, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except InvalidHeader as e:
            return BaseResponse(resEnum=ResponseEnum.Invalid_Header, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except InvalidProxyURL as e:
            return BaseResponse(resEnum=ResponseEnum.Invalid_Proxy_URL, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except ConnectionError as e:
            return BaseResponse(resEnum=ResponseEnum.ConnectionError, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except ConnectTimeout as e:
            return BaseResponse(resEnum=ResponseEnum.ConnectionTimeout, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except Exception as e:
            return BaseResponse(resEnum=ResponseEnum.InternalError, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))

    @staticmethod
    @access_log()
    def post(url: str, headers=None, json_data=None, encoding=None, timeout=30):
        """
        同步post
        :param url: 请求的URL
        :param headers: 请求头
        :param timeout: 超时时间，默认为10秒
        :param json_data: 要发送的JSON数据
        :param encoding: 编码方式
        :return: 解析后的JSON响应
        """
        try:
            new_headers = RequestUtil.builder_headers(headers)
            result = requests.post(url, headers=new_headers, json=json_data, timeout=timeout)
            if encoding:
                result.encoding = encoding
            text = result.text
            return BaseResponse(data=json.loads(text), resEnum=ResponseEnum.OK, msg=f"请地址url:{url}")
        except ReadTimeout as e:
            return BaseResponse(resEnum=ResponseEnum.RequestTimeout, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except TooManyRedirects as e:
            return BaseResponse(resEnum=ResponseEnum.Too_Many_Redirects, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except URLRequired as e:
            return BaseResponse(resEnum=ResponseEnum.InvalidRequest, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except InvalidURL as e:
            return BaseResponse(resEnum=ResponseEnum.InvalidURI, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except InvalidHeader as e:
            return BaseResponse(resEnum=ResponseEnum.Invalid_Header, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except InvalidProxyURL as e:
            return BaseResponse(resEnum=ResponseEnum.Invalid_Proxy_URL, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except ConnectionError as e:
            return BaseResponse(resEnum=ResponseEnum.ConnectionError, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except ConnectTimeout as e:
            return BaseResponse(resEnum=ResponseEnum.ConnectionTimeout, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))
        except Exception as e:
            return BaseResponse(resEnum=ResponseEnum.InternalError, msg=f"请地址url:{url}",
                                exceptionStack=format_exception_msg(e))

    @staticmethod
    @access_log()
    def builder_headers(headers=None):

        """
        x-request-id:7d5e4ea0-e2dd-48bb-a8bb-91a9fd99841d-1749629043413
        x-user-id:1904456484926791700
            "x-user-id": "test_user",  # 添加用户ID
            "x-forwarded-for": "127.0.0.1",  # 添加有效的IP地址
            "x-request-id": "test_request_id",
            "x-trace-id": "test_trace_id"
        """
        try:
            # 如果是request.headers
            if isinstance(headers, Headers):
                headers = MutableHeaders(headers)


            request_id: str = headers.get(SthgCommonConstants.header_x_request_id_key)
            if request_id is None or len(request_id) == 0:
                headers[SthgCommonConstants.header_x_request_id_key] = ContextVar(
                    SthgCommonConstants.context_request_id_key, default="").get()

            user_id: str = headers.get(SthgCommonConstants.header_x_user_id_key)
            if user_id is None or len(user_id) == 0:
                user_id = headers.get(SthgCommonConstants.header_user_id_key)
            if user_id is None or len(user_id) == 0:
                headers[SthgCommonConstants.header_x_user_id_key] = ContextVar(SthgCommonConstants.context_user_id_key,
                                                                               default="").get()
                headers[SthgCommonConstants.header_user_id_key] = ContextVar(SthgCommonConstants.context_user_id_key,
                                                                             default="").get()

            trace_id: str = headers.get(SthgCommonConstants.header_x_trace_id_key)
            if trace_id is None or len(trace_id) == 0:
                headers[SthgCommonConstants.header_x_trace_id_key] = ContextVar(SthgCommonConstants.context_trace_id_key,
                                                                                default="").get()

            x_guid: str = headers.get(SthgCommonConstants.header_x_guid_id_key)
            if x_guid is None or len(x_guid) == 0:
                headers[SthgCommonConstants.header_x_guid_id_key] = ContextVar(SthgCommonConstants.context_guid_id_key,
                                                                               default="").get()

            return headers
        except Exception as e:
            LoggerUtil.error_log(f"builder_headers error "+str(e))
            return headers


if __name__ == "__main__":
    # # 测试http_get方法 - 成功案例
    # print("测试http_get - 成功案例:")
    # get_result = RequestUtil.get(
    #     url="https://httpbin.org/get",
    #     params={"test": "value"},
    #     headers={"User-Agent": "TestClient"}
    # )
    # print(f"状态码: {get_result.code}")
    # print(f"业务消息: {get_result.busiMsg}")
    # if get_result.code == ResponseEnum.OK.getHttpCode:
    #     print(f"响应数据: {get_result.data}")
    #     print(f"请求ID: {get_result.requestId}")
    # else:
    #     print(f"错误信息: {get_result.exceptionStack}")
    #
    # # 测试http_get方法 - 超时案例
    # print("\n测试http_get - 超时案例:")
    # timeout_result = RequestUtil.get(
    #     url="https://httpbin.org/delay/5",  # 这个端点会延迟5秒响应
    #     timeout=1  # 设置1秒超时
    # )
    # print(f"状态码: {timeout_result.code}")
    # print(f"业务消息: {timeout_result.busiMsg}")
    # print(f"错误堆栈: {timeout_result.exceptionStack}")
    #
    # 测试http_post方法 - 成功案例
    # print("\n测试http_post - 成功案例:")
    # post_data = {"key": "value", "number": 42}
    # post_result = RequestUtil.post(
    #     url="https://httpbin.org/post",
    #     json_data=post_data,
    #     headers={"Content-Type": "application/json"}
    # )
    # print(f"状态码: {post_result.code}")
    # print(f"业务消息: {post_result.busiMsg}")
    # if post_result.code == ResponseEnum.OK.getHttpCode:
    #     print(f"响应数据: {post_result.data}")
    #     print(f"请求ID: {post_result.requestId}")
    # else:
    #     print(f"错误信息: {post_result.exceptionStack}")
    #
    # # 测试http_post方法 - 错误URL案例
    # print("\n测试http_post - 错误URL案例:")
    # error_result = RequestUtil.post(
    #     url="https://invalid-domain-xyz.example/api",
    #     json_data={"test": "data"}
    # )
    # print(f"状态码: {error_result.code}")
    # print(f"业务消息: {error_result.busiMsg}")
    # print(f"错误堆栈: {error_result.exceptionStack}")
    #
    # # 测试http_post方法 - 无效JSON响应
    # print("\n测试http_post - 无效JSON响应:")
    # invalid_json_result = RequestUtil.post(
    #     url="https://httpbin.org/html",  # 返回HTML内容而不是JSON
    #     headers={"Accept": "application/json"}  # 故意要求JSON
    # )
    # print(f"状态码: {invalid_json_result.code}")
    # print(f"业务消息: {invalid_json_result.busiMsg}")
    # print(f"错误堆栈: {invalid_json_result.exceptionStack}")

    bode: dict = {
        "current_node_id": "63a2d06a-f8d3-4cf3-a65c-20f4b448a019",
        "graph": {
            "edges": [
                {
                    "shape": "edge",
                    "attrs": {
                        "line": {
                            "stroke": "#537FDF",
                            "strokeWidth": 1,
                            "strokeDasharray": 5
                        }
                    },
                    "id": "525a7895-f2b7-4303-822b-80b7bdd13130",
                    "router": {
                        "name": "manhattan"
                    },
                    "connector": {
                        "name": "rounded"
                    },
                    "data": {
                        "source": "a5009321-1d7e-4ff5-831d-638d2c261bbc",
                        "target": "90fde588-881b-4404-93c1-0fc7b31711dc"
                    },
                    "source": "a5009321-1d7e-4ff5-831d-638d2c261bbc",
                    "target": "90fde588-881b-4404-93c1-0fc7b31711dc",
                    "zIndex": 6
                },
                {
                    "shape": "edge",
                    "attrs": {
                        "line": {
                            "stroke": "#537FDF",
                            "strokeWidth": 1,
                            "strokeDasharray": 5
                        }
                    },
                    "id": "ab879292-6122-4943-96fa-93e2e4af7a95",
                    "router": {
                        "name": "manhattan"
                    },
                    "connector": {
                        "name": "rounded"
                    },
                    "data": {
                        "source": "90fde588-881b-4404-93c1-0fc7b31711dc",
                        "target": "f743f047-b8dc-4cf1-a047-c1fae4131357"
                    },
                    "source": "90fde588-881b-4404-93c1-0fc7b31711dc",
                    "target": "f743f047-b8dc-4cf1-a047-c1fae4131357",
                    "zIndex": 7
                },
                {
                    "shape": "edge",
                    "attrs": {
                        "line": {
                            "stroke": "#537FDF",
                            "strokeWidth": 1,
                            "strokeDasharray": 5
                        }
                    },
                    "id": "1bebc19b-12ec-4eb0-88bf-724111325bde",
                    "router": {
                        "name": "manhattan"
                    },
                    "connector": {
                        "name": "rounded"
                    },
                    "data": {
                        "source": "63a2d06a-f8d3-4cf3-a65c-20f4b448a019",
                        "target": "f743f047-b8dc-4cf1-a047-c1fae4131357"
                    },
                    "source": "63a2d06a-f8d3-4cf3-a65c-20f4b448a019",
                    "target": "f743f047-b8dc-4cf1-a047-c1fae4131357",
                    "zIndex": 8
                },
                {
                    "shape": "edge",
                    "attrs": {
                        "line": {
                            "stroke": "#537FDF",
                            "strokeWidth": 1,
                            "strokeDasharray": 5
                        }
                    },
                    "id": "a048f5f1-8a0f-49b8-b582-951c7d2ab46a",
                    "router": {
                        "name": "manhattan"
                    },
                    "connector": {
                        "name": "rounded"
                    },
                    "data": {
                        "source": "f743f047-b8dc-4cf1-a047-c1fae4131357",
                        "target": "5162970b-8786-487b-94d0-66349dfedb92"
                    },
                    "source": "f743f047-b8dc-4cf1-a047-c1fae4131357",
                    "target": "5162970b-8786-487b-94d0-66349dfedb92",
                    "zIndex": 9
                }
            ],
            "nodes": [
                {
                    "position": {
                        "x": 100,
                        "y": 200
                    },
                    "size": {
                        "width": 240,
                        "height": 73
                    },
                    "view": "vue-shape-view",
                    "shape": "data-node",
                    "ports": {
                        "groups": {
                            "right": {
                                "position": "right",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "right"
                                }
                            },
                            "left": {
                                "position": "left",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "left"
                                }
                            }
                        },
                        "items": [
                            {
                                "id": "63a2d06a-f8d3-4cf3-a65c-20f4b448a019-out",
                                "group": "right"
                            }
                        ]
                    },
                    "id": "63a2d06a-f8d3-4cf3-a65c-20f4b448a019",
                    "data": {
                        "key": "dd_zhendi_status",
                        "columns": [
                            {
                                "columnName": "count_zdzb",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None,
                                "length": 1024
                            },
                            {
                                "columnName": "name_ddfsj",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None
                            },
                            {
                                "columnName": "count_ddfsj",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None,
                                "length": 1020
                            },
                            {
                                "columnName": "name_zdzb",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None
                            },
                            {
                                "columnName": "action_name",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None
                            }
                        ],
                        "row_count": 27,
                        "create_time": "2025-06-10T01:52:34",
                        "update_time": "2025-06-10T01:52:35",
                        "table_size": "0.00 MB",
                        "comment": None,
                        "description": "",
                        "isCheckbox": True,
                        "name": None,
                        "table": "dd_zhendi_status",
                        "data_type": "dataset",
                        "node_type": "INPUT",
                        "showMenu": False
                    },
                    "zIndex": 1
                },
                {
                    "position": {
                        "x": 100,
                        "y": 300
                    },
                    "size": {
                        "width": 240,
                        "height": 73
                    },
                    "view": "vue-shape-view",
                    "shape": "data-node",
                    "ports": {
                        "groups": {
                            "right": {
                                "position": "right",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "right"
                                }
                            },
                            "left": {
                                "position": "left",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "left"
                                }
                            }
                        },
                        "items": [
                            {
                                "id": "a5009321-1d7e-4ff5-831d-638d2c261bbc-out",
                                "group": "right"
                            }
                        ]
                    },
                    "id": "a5009321-1d7e-4ff5-831d-638d2c261bbc",
                    "data": {
                        "key": "xy_refined_result",
                        "columns": [
                            {
                                "columnName": "task_id",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None,
                                "length": 128
                            },
                            {
                                "columnName": "target_name",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None,
                                "length": 1020
                            },
                            {
                                "columnName": "target_id",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None,
                                "length": 1020
                            },
                            {
                                "columnName": "start_time",
                                "columnType": "Timestamp",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None
                            },
                            {
                                "columnName": "satellite",
                                "columnType": "String",
                                "is_nullable": True,
                                "is_primary_key": False,
                                "is_unique": False,
                                "is_indexed": False,
                                "comment": None,
                                "length": 1020
                            },
                            {
                                "columnName": "data_json",
                                "columnType": "String",
                            },
                            {
                                "columnName": "type",
                                "columnType": "Integer",
                            },
                            {
                                "columnName": "target_type",
                                "columnType": "Integer",
                            }
                        ],
                        "row_count": 61,
                        "create_time": "2025-06-07T19:51:11",
                        "update_time": "2025-06-07T22:27:28",
                        "table_size": "0.00 MB"
                    },
                    "zIndex": 2
                },
                {
                    "position": {
                        "x": 450,
                        "y": 300
                    },
                    "size": {
                        "width": 240,
                        "height": 73
                    },
                    "view": "vue-shape-view",
                    "shape": "data-node",
                    "ports": {
                        "groups": {
                            "right": {
                                "position": "right",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "right"
                                }
                            },
                            "left": {
                                "position": "left",
                                "attrs": {
                                    "circle": {
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "left"
                                }
                            }
                        },
                        "items": [
                            {
                                "id": "90fde588-881b-4404-93c1-0fc7b31711dc-input",
                                "group": "left"
                            },
                            {
                                "id": "90fde588-881b-4404-93c1-0fc7b31711dc-out",
                                "group": "right"
                            }
                        ]
                    },
                    "id": "90fde588-881b-4404-93c1-0fc7b31711dc",
                    "data": {
                        "data_type": "transform",
                        "node_type": "TRANSFORM",
                        "key": "转换",
                        "transformer_list": [
                            {
                                "alias": "",
                                "function_name": "JSONParsing",
                                "table_name": "xy_refined_result",
                                "argument_list": [
                                    {
                                        "param_name": "",
                                        "is_field": False,
                                        "value": "[\n    {\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 1,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },\n{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 2,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 3,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 4,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    }\n,{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 1,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    }\n]"
                                    },
                                    {
                                        "param_name": "",
                                        "is_field": False,
                                        "value": "[{\"name\":\"name1\",\"types\":\"String\"},{\"name\":\"type1\",\"types\":\"String\"},{\"name\":\"unit1\",\"types\":\"String\"},{\"name\":\"count1\",\"types\":\"String\"},{\"name\":\"changecount1\",\"types\":\"String\"},{\"name\":\"name2\",\"types\":\"String\"},{\"name\":\"type2\",\"types\":\"String\"},{\"name\":\"unit2\",\"types\":\"String\"},{\"name\":\"count2\",\"types\":\"String\"},{\"name\":\"changecount2\",\"types\":\"String\"}]"
                                    }
                                ]
                            }
                        ],
                        "allForData": [
                            {
                                "title": "JSONParsing",
                                "sharedData": {
                                    "rid": "ri.ontology.main.transform_operator.29e92254-3818-479b-a4af-da7d021e1b60",
                                    "create_uid": "123",
                                    "ctime": "2025-05-27T12:09:34",
                                    "utime": "2025-05-27T12:09:34",
                                    "name": "json解析",
                                    "en_name": "JSONParsing",
                                    "type": "数据准备",
                                    "description": "对 JSON 格式的数据进行解析",
                                    "input_params": {},
                                    "output_params": {},
                                    "user_params": {},
                                    "icon": {},
                                    "tableName": "xy_refined_result",
                                    "selectedTags": [
                                        {
                                            "label": "task_id",
                                            "value": "task_id",
                                            "typeName": "STRING"
                                        },
                                        {
                                            "label": "target_name",
                                            "value": "target_name",
                                            "typeName": "STRING"
                                        },
                                        {
                                            "label": "target_id",
                                            "value": "target_id",
                                            "typeName": "STRING"
                                        },
                                        {
                                            "label": "start_time",
                                            "value": "start_time",
                                            "typeName": "TIMESTAMP(3)"
                                        },
                                        {
                                            "label": "satellite",
                                            "value": "satellite",
                                            "typeName": "STRING"
                                        },
                                        {
                                            "label": "data_json",
                                            "value": "data_json",
                                            "typeName": "STRING"
                                        },
                                        {
                                            "label": "type",
                                            "value": "type",
                                            "typeName": "INT"
                                        },
                                        {
                                            "label": "target_type",
                                            "value": "target_type",
                                            "typeName": "INT"
                                        }
                                    ],
                                    "treeObj": [
                                        {
                                            "rid": "ri.ontology.main.transform_operator.b1d7478a-dd6f-4339-97b3-8f09b61004ba",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:17",
                                            "utime": "2025-05-27T12:09:17",
                                            "name": "数值乘",
                                            "en_name": "NumericMultiplication",
                                            "type": "数值",
                                            "description": "两个数相乘",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "左",
                                                            "title": "乘数"
                                                        },
                                                        {
                                                            "heading": "右",
                                                            "title": "被乘数"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.c1f2d0b8-c756-4df4-9d9c-6c72d44c029e",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:13",
                                            "utime": "2025-05-27T12:09:13",
                                            "name": "数值加",
                                            "en_name": "NumericAddition",
                                            "type": "数值",
                                            "description": "两个数相加",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "左",
                                                            "title": "加数"
                                                        },
                                                        {
                                                            "heading": "右",
                                                            "title": "被加数"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.52e45029-ca58-436a-883d-7e02ff4367a9",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:19",
                                            "utime": "2025-05-27T12:09:19",
                                            "name": "数值除",
                                            "en_name": "NumericDivision",
                                            "type": "数值",
                                            "description": "两个数相除",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "左",
                                                            "title": "除数"
                                                        },
                                                        {
                                                            "heading": "右",
                                                            "title": "被除数"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.4e075471-0cae-4689-b1f5-509631741a36",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:15",
                                            "utime": "2025-05-27T12:09:15",
                                            "name": "数值减",
                                            "en_name": "NumericSubtraction",
                                            "type": "数值",
                                            "description": "两个数相减",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "左",
                                                            "title": "减数"
                                                        },
                                                        {
                                                            "heading": "右",
                                                            "title": "被减数"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.6483c19e-2f48-4143-9f74-c04519f2c69e",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:21",
                                            "utime": "2025-05-27T12:09:21",
                                            "name": "连接字符串",
                                            "en_name": "ConcatenateStrings",
                                            "type": "字符串",
                                            "description": "将一个字符串列表与指定的连接符连接起来",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "连接符",
                                                            "title": "字符串中间的连接符"
                                                        },
                                                        {
                                                            "heading": "字符串列表",
                                                            "title": "需要连接的字符串列表"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.40a600ad-ea33-4110-afca-d3f87c55d179",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:23",
                                            "utime": "2025-05-27T12:09:23",
                                            "name": "哈希sha256",
                                            "en_name": "HashSHA256",
                                            "type": "字符串",
                                            "description": "对字符串进行 SHA256 哈希运算",
                                            "example": None,
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.0ddd3f59-177e-4d00-95f6-fa56cf0fe4b0",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:10",
                                            "utime": "2025-05-27T12:09:10",
                                            "name": "选择列",
                                            "en_name": "SelectColumn",
                                            "type": "常用",
                                            "description": "从输入数据集选择指定列",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "要选择的列",
                                                            "title": "要选择的列的列表"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.f1e7292f-8237-4acd-ba26-d1a3079b5b66",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:08",
                                            "utime": "2025-05-27T12:09:08",
                                            "name": "删除列",
                                            "en_name": "DeleteColumn",
                                            "type": "常用",
                                            "description": "删除指定列，转换输入数据集",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "要删除的列",
                                                            "title": "要删除的列的列表"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.8122d450-ffac-4cb0-b81d-45a9bf6f1f72",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:06",
                                            "utime": "2025-05-27T12:09:06",
                                            "name": "cast函数转换",
                                            "en_name": "CastFunctionConversion",
                                            "type": "转换",
                                            "description": "将值、列或表达式转换成另一种类型，包含日期时间转换以及反之转换",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "表达式",
                                                            "title": "转换表达式"
                                                        },
                                                        {
                                                            "heading": "类型",
                                                            "title": "需要转换成的类型"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.0a3abf4c-9de6-4f0d-a788-cf187073d28b",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:30",
                                            "utime": "2025-05-27T12:09:30",
                                            "name": "Ontology GeoPoint转GeoPoint",
                                            "en_name": "OntologyGeoPointToGeoPoint",
                                            "type": "地理空间",
                                            "description": "将一个Ontology GeoPoint转换为一个常规GeoPoint。Ontology GeoPoint是格式为\"{lat},{lon}\"的字符串，其中-90 <= lat <= 90 且 -180 <= lon <= 180。常规GeoPoint是格式为{\"longitude\": {long},\"latitude\": {lat}}的结构。",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "表达式",
                                                            "title": "要转换的 Ontology GeoPoint"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.2b700568-102e-404c-b814-934887357670",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:28",
                                            "utime": "2025-05-27T12:09:28",
                                            "name": "获取H3索引",
                                            "en_name": "GetH3Index",
                                            "type": "地理空间",
                                            "description": "将GeoPoint转换为给定分辨率的H3索引。对于分辨率<0或>15返回null。",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "GeoPoint",
                                                            "title": "要转换为 H3 索引的 GeoPoint (lon,lat)"
                                                        },
                                                        {
                                                            "heading": "Resolution",
                                                            "title": "H3 网格分辨率，范围在 0 到 15 之间（包括 0 和 15）"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.81088269-9e4c-4fbd-b674-55159a11323b",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:25",
                                            "utime": "2025-05-27T12:09:25",
                                            "name": "构建Geopoint列",
                                            "en_name": "BuildGeopointColumn",
                                            "type": "地理空间",
                                            "description": "从纬度和经度列构建 GeoPoint 列。验证纬度参数是否在-90和90之间（包括边界），以及经度参数是否在-180和180之间（包括边界）；如果不在范围内，则返回空值。",
                                            "example": [
                                                {
                                                    "key": "表达式类别",
                                                    "children": [
                                                        {
                                                            "heading": "类别",
                                                            "title": "地理空间"
                                                        }
                                                    ]
                                                },
                                                {
                                                    "key": "声明的参数",
                                                    "children": [
                                                        {
                                                            "heading": "纬度",
                                                            "title": "纬度列"
                                                        },
                                                        {
                                                            "heading": "经度",
                                                            "title": "经度列"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        },
                                        {
                                            "rid": "ri.ontology.main.transform_operator.d6c46310-74c6-403c-93db-02569fa5b2e1",
                                            "create_uid": "123",
                                            "ctime": "2025-05-27T12:09:32",
                                            "utime": "2025-05-27T12:09:32",
                                            "name": "GeoPoint转Ontology GeoPoint",
                                            "en_name": "GeoPointToOntologyGeoPoint",
                                            "type": "地理空间",
                                            "description": "将 GeoPoint 转换为字符串，以便 Ontology 接受地理索引列（地理哈希类型列）。Ontology GeoPoint 是格式为 '{lat},{lon}' 的字符串，其中 -90 <= lat <= 90 且 -180 <= lon <= 180。",
                                            "example": [
                                                {
                                                    "key": "参数",
                                                    "children": [
                                                        {
                                                            "heading": "表达式",
                                                            "title": "要转换的 Ontology GeoPoint"
                                                        }
                                                    ]
                                                }
                                            ],
                                            "input_params": {},
                                            "output_params": {},
                                            "user_params": {},
                                            "icon": {}
                                        }
                                    ]
                                },
                                "allList": {
                                    "expression": "json解析",
                                    "textarea": "[\n    {\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 1,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },\n{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 2,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 3,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 4,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    }\n,{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 1,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    }\n]",
                                    "Json": "[\n    {\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 1,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },\n{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 2,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 3,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    },{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 4,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    }\n,{\n        \"name1\": \"导D发射架\",\n        \"type1\": \"发射架\",\n        \"unit1\": \"6\",\n        \"count1\": 1,\n        \"changecount1\": 1,\n        \"name2\": \"制导装备\",\n        \"type2\": \"\",\n        \"unit2\": \"1\",\n        \"count2\": 1,\n        \"changecount2\": 0\n    }\n]",
                                    "Schema": "struct",
                                    "dataSet": [
                                        {
                                            "name": "name1",
                                            "types": "String"
                                        },
                                        {
                                            "name": "type1",
                                            "types": "String"
                                        },
                                        {
                                            "name": "unit1",
                                            "types": "String"
                                        },
                                        {
                                            "name": "count1",
                                            "types": "String"
                                        },
                                        {
                                            "name": "changecount1",
                                            "types": "String"
                                        },
                                        {
                                            "name": "name2",
                                            "types": "String"
                                        },
                                        {
                                            "name": "type2",
                                            "types": "String"
                                        },
                                        {
                                            "name": "unit2",
                                            "types": "String"
                                        },
                                        {
                                            "name": "count2",
                                            "types": "String"
                                        },
                                        {
                                            "name": "changecount2",
                                            "types": "String"
                                        }
                                    ]
                                },
                                "newsCol": "",
                                "dataForName": "xy_refined_result"
                            },
                            {
                                "title": "Transform",
                                "sharedData": {},
                                "allList": [],
                                "newsCol": "",
                                "dataForName": "xy_refined_result"
                            }
                        ],
                        "columns": [
                            {
                                "columnName": "task_id",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "target_name",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "target_id",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "start_time",
                                "columnType": "TIMESTAMP(3)",
                                "objectType": "Object"
                            },
                            {
                                "columnName": "satellite",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "data_json",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "type",
                                "columnType": "INT",
                                "objectType": "Integer"
                            },
                            {
                                "columnName": "target_type",
                                "columnType": "INT",
                                "objectType": "Integer"
                            },
                            {
                                "columnName": "name1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "type1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "unit1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "count1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "changecount1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "name2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "type2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "unit2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "count2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "changecount2",
                                "columnType": "STRING",
                                "objectType": "String"
                            }
                        ],
                        "rowCount": 18,
                        "left_node": "a5009321-1d7e-4ff5-831d-638d2c261bbc",
                        "showMenu": False
                    },
                    "zIndex": 3
                },
                {
                    "position": {
                        "x": 800,
                        "y": 300
                    },
                    "size": {
                        "width": 240,
                        "height": 73
                    },
                    "view": "vue-shape-view",
                    "shape": "data-node",
                    "ports": {
                        "groups": {
                            "right": {
                                "position": "right",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "right"
                                }
                            },
                            "left": {
                                "position": "left",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "left"
                                }
                            }
                        },
                        "items": [
                            {
                                "id": "f743f047-b8dc-4cf1-a047-c1fae4131357-input",
                                "group": "left"
                            },
                            {
                                "id": "f743f047-b8dc-4cf1-a047-c1fae4131357-out",
                                "group": "right"
                            }
                        ]
                    },
                    "id": "f743f047-b8dc-4cf1-a047-c1fae4131357",
                    "data": {
                        "key": "连接",
                        "node_type": "JOIN",
                        "joinType": "inner",
                        "match_condition": [
                            {
                                "source_table": "",
                                "source_id": "90fde588-881b-4404-93c1-0fc7b31711dc",
                                "source_column": "name1",
                                "target_table": "dd_zhendi_status",
                                "target_id": "",
                                "target_column": "name_ddfsj"
                            },
                            {
                                "source_table": "",
                                "source_id": "90fde588-881b-4404-93c1-0fc7b31711dc",
                                "source_column": "count1",
                                "target_table": "dd_zhendi_status",
                                "target_id": "",
                                "target_column": "count_ddfsj"
                            },
                            {
                                "source_table": "",
                                "source_id": "90fde588-881b-4404-93c1-0fc7b31711dc",
                                "source_column": "name2",
                                "target_table": "dd_zhendi_status",
                                "target_id": "",
                                "target_column": "name_zdzb"
                            },
                            {
                                "source_table": "",
                                "source_id": "90fde588-881b-4404-93c1-0fc7b31711dc",
                                "source_column": "count2",
                                "target_table": "dd_zhendi_status",
                                "target_id": "",
                                "target_column": "count_zdzb"
                            }
                        ],
                        "select_source_column": [],
                        "select_target_column": [],
                        "left_table": "转换",
                        "right_table": "dd_zhendi_status",
                        "leftNodeId": "90fde588-881b-4404-93c1-0fc7b31711dc",
                        "rightNodeId": "63a2d06a-f8d3-4cf3-a65c-20f4b448a019",
                        "prefix": "",
                        "hiddenAdvanced": False,
                        "data_type": "join",
                        "columns": [
                            {
                                "columnName": "task_id",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "target_name",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "target_id",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "start_time",
                                "columnType": "TIMESTAMP(3)",
                                "objectType": "Object"
                            },
                            {
                                "columnName": "satellite",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "data_json",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "type",
                                "columnType": "INT",
                                "objectType": "Integer"
                            },
                            {
                                "columnName": "target_type",
                                "columnType": "INT",
                                "objectType": "Integer"
                            },
                            {
                                "columnName": "name1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "type1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "unit1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "count1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "changecount1",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "name2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "type2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "unit2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "count2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "changecount2",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "count_zdzb",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "name_ddfsj",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "count_ddfsj",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "name_zdzb",
                                "columnType": "STRING",
                                "objectType": "String"
                            },
                            {
                                "columnName": "action_name",
                                "columnType": "STRING",
                                "objectType": "String"
                            }
                        ],
                        "rowCount": 23,
                        "showMenu": False
                    },
                    "zIndex": 4
                },
                {
                    "position": {
                        "x": 1150,
                        "y": 300
                    },
                    "size": {
                        "width": 240,
                        "height": 73
                    },
                    "view": "vue-shape-view",
                    "shape": "data-node",
                    "ports": {
                        "groups": {
                            "right": {
                                "position": "right",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "right"
                                }
                            },
                            "left": {
                                "position": "left",
                                "attrs": {
                                    "circle": {
                                        "magnet": True,
                                        "stroke": "#8f8f8f",
                                        "r": 5
                                    }
                                },
                                "label": {
                                    "position": "left"
                                }
                            }
                        },
                        "items": [
                            {
                                "id": "5162970b-8786-487b-94d0-66349dfedb92-input",
                                "group": "left"
                            }
                        ]
                    },
                    "id": "5162970b-8786-487b-94d0-66349dfedb92",
                    "data": {
                        "key": "Xy_0610_02",
                        "data_type": "output",
                        "pipeline_data_type": "dataset",
                        "table": "Xy_0610_02",
                        "output_name": "Xy_0610_02",
                        "columns": [
                            {
                                "columnName": "task_id",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "task_id",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "target_name",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "target_name",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "target_id",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "target_id",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "start_time",
                                "columnType": "TIMESTAMP(3)",
                                "objectType": "Object",
                                "field_name": "start_time",
                                "field_type": "TIMESTAMP(3)"
                            },
                            {
                                "columnName": "satellite",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "satellite",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "data_json",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "data_json",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "type",
                                "columnType": "INT",
                                "objectType": "Integer",
                                "field_name": "type",
                                "field_type": "INT"
                            },
                            {
                                "columnName": "target_type",
                                "columnType": "INT",
                                "objectType": "Integer",
                                "field_name": "target_type",
                                "field_type": "INT"
                            },
                            {
                                "columnName": "name1",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "name1",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "type1",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "type1",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "unit1",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "unit1",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "count1",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "count1",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "changecount1",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "changecount1",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "name2",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "name2",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "type2",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "type2",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "unit2",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "unit2",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "count2",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "count2",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "changecount2",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "changecount2",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "count_zdzb",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "count_zdzb",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "name_ddfsj",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "name_ddfsj",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "count_ddfsj",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "count_ddfsj",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "name_zdzb",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "name_zdzb",
                                "field_type": "STRING"
                            },
                            {
                                "columnName": "action_name",
                                "columnType": "STRING",
                                "objectType": "String",
                                "field_name": "action_name",
                                "field_type": "STRING"
                            }
                        ],
                        "left_node": "f743f047-b8dc-4cf1-a047-c1fae4131357",
                        "node_type": "OUTPUT"
                    },
                    "zIndex": 5
                }
            ]
        }
    }
    # 测试http trace_id、user_id 透传 - 成功案例
    print("测试xtrace_id_result - 成功案例:")

    headers = {"User-Agent": "TestClient", "x-trace-id": "x-trace-id-11111111111111",
               'Content-Type': 'application/json',
               "x-user-id": "x-user-id-222222222222222", "x-request-id": "x-request-id-zzzzzzzzzzzzzzzzzzzzzzzzzzzz"}
    xtrace_id_result = RequestUtil.post(
        url="http://192.168.1.245:31045/api/pipelines/v1/pipelines/currentNode/data",
        json_data=bode,
        headers=headers
    )
    # 查看实际发送的请求头

    print(f"状态码: {xtrace_id_result.code}")
    print(f"业务消息: {xtrace_id_result.busiMsg}")
    print(f"错误堆栈: {xtrace_id_result.exceptionStack}")

    # 测试http trace_id、user_id 透传 - 成功案例
    print("测试xtrace_id_result - 失败案例:")
    bode["current_node_id"] = ""
    xtrace_id_result = RequestUtil.post(
        url="http://192.168.1.245:31045/api/pipelines/v1/pipelines/currentNode/data",
        json_data=bode,
        headers=headers
    )

    print(f"状态码: {xtrace_id_result.code}")
    print(f"业务消息: {xtrace_id_result.busiMsg}")
    print(f"错误堆栈: {xtrace_id_result.exceptionStack}")



import datetime
from time import sleep
from typing import Optional


from sthg_common_base.response.exception import CustomException
from sthg_common_base.response.httpCodeEnum import ResponseEnum
from sthg_common_base.response.response import BaseResponse
from sthg_common_base.utils.log_wrapper import service_log
from pydantic import BaseModel,Field

from sthg_common_base.utils.request_util import RequestUtil


# 注册自定义异常
class AccessDeniedError(Exception):
    pass

class Person(BaseModel):
    ID: Optional[str] = Field(None, description="HTTP 状态码")      # 声明字段类型
    name: Optional[str] = Field(None, description="HTTP 状态码")       # 声明字段类型

    # 不需要自定义 __init__，Pydantic 会自动处理

    def get_id(self):
        return self.ID

    def get_name(self):
        return self.name

    def set_id(self, new_id):
        self.ID = new_id

    def set_name(self, new_name):
        self.name = new_name

    def __str__(self):
        return f"Person(ID={str(self.ID)}, name='{self.name}')"


class Person2():
    def __init__(self, ID, name):
        self.ID = ID
        self.name = name

    def get_id(self):
        return self.ID

    def get_name(self):
        return self.name

    def set_id(self, new_id):
        self.ID = new_id

    def set_name(self, new_name):
        self.name = new_name

    def __str__(self):
        return f"Person(ID={self.ID}, name='{self.name}')"



@service_log(printReq=True,printResp=True)
def get_Object333(id:str)->Person:
    per = Person2("1","1231")

    return per

@service_log(printReq=True,printResp=True)
def get_Object(id:str)->BaseResponse:
    per = Person(ID="1",name="1231")
    RequestUtil.get("http://www.baidu.com")
    RequestUtil.post("http://www.baidu.com")
    return  BaseResponse(ResponseEnum.OK,per)

@service_log(printReq=True,printResp=True)
def get_Object_rasie1(id:str)->Person:
    per = Person(ID="1",name="1231")
    raise AccessDeniedError

@service_log(printReq=True,printResp=True)
def get_baseRes(id:str)->BaseResponse:
    per = Person(ID="2",name="anne2")
    result = BaseResponse(ResponseEnum.OK,per)

    return result


@service_log(printReq=True, printResp=True)
def get_Object_Exception(id: str) -> BaseResponse:
    per = Person(ID="2",name="anne2")

    raise CustomException(ResponseEnum.InternalError,None)

    return result


@service_log(printReq=True, printResp=True,throwException=False)
def get_Object_Exception2(id: str) -> BaseResponse:
    per = Person(ID="2",name="anne2")

    raise CustomException(ResponseEnum.InternalError,None)

    return result

@service_log(printReq=True, printResp=True)
def get_Object_MaxRt(id: str) -> BaseResponse:
    per = Person(ID="2",name="anne2")

    result = BaseResponse(ResponseEnum.OK, per)
    sleep(1)
    return result

@service_log(printReq=True,printResp=True)
async def get_Object_ansyc(id:str)->Person:
    per = Person(ID="2",name="anne2")
    return per

@service_log(printReq=True,printResp=True)
async def get_baseRes_ansyc(id:str)->BaseResponse:
    per = Person(ID="2",name="anne2")
    result = BaseResponse(ResponseEnum.OK,per)

    return result

@service_log(printReq=True, printResp=True)
async def get_Object_Exception_ansyc(id: str) -> BaseResponse:
    per = Person(ID="2",name="anne2")

    raise CustomException(ResponseEnum.InternalError,None)

    return result


@service_log(printReq=True, printResp=True,throwException=False)
async def get_Object_Exception2_ansyc(id: str) -> BaseResponse:
    per = Person(ID="2",name="anne2")

    raise CustomException(ResponseEnum.InternalError,None)

    return result

@service_log(printReq=True, printResp=True)
async def get_Object_MaxRt_ansyc(id: str) -> BaseResponse:
    per = Person(ID="2",name="anne2")

    result = BaseResponse(ResponseEnum.OK, per)
    sleep(1)
    return result



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Author  ：DongQing
@File    ：exception.py
@Time    ：2025/3/31
@Desc    ：
"""
__all__ = [
    'BaseException',
    'CustomException',
    'register_exception_handlers'
]

from fastapi import FastAPI, Request
from fastapi.exceptions import HTTPException, RequestValidationError, StarletteHTTPException
from pydantic import ValidationError
from starlette.responses import JSONResponse

from sthg_common_base.response.httpCodeEnum import HttpStatus, ResponseEnum
from sthg_common_base.utils.constants import Constants
from sthg_common_base.utils.log_util import Logger, LoggerUtil, TraceID, MDC
import inspect
import os
import traceback

ERROR_TYPE_MAPPING = {
    "value_error.number.not_ge": "{},值不能小于:{}",
    "value_error.list.min_items": "{},元素个数至少为:{}",
    "value_error.str": "{},元素个数至少为:{}",
    "value_error.missing": "字段必填",
    "type_error.integer": "必须是整数类型",
    "value_error.number.not_gt": "必须大于 {limit_value}",
}

KeyErrorChineseDict = {
    "": ""
}


# 定义一个自定义异常类
class BaseException(Exception):
    code: int
    busiMsg: str
    busiCode: str
    exceptionStack: str
    requestId: str

    def __init__(self, resEnum: ResponseEnum, msg: str = None, *args):
        if msg and args:
            msg = msg.format(*args)
        self.code = resEnum.getHttpCode
        self.busiCode = resEnum.getBusiCode
        if msg:
            self.busiMsg = f"{resEnum.getBusiMsg},{msg}"
        else:
            self.busiMsg = resEnum.getBusiMsg

        super().__init__(self.busiMsg)

    def __call__(self) -> JSONResponse:
        return JSONResponse(
            status_code=self.code,
            content={
                "code": self.code,
                "busiCode": self.busiCode,
                "busiMsg": self.busiMsg,
                "data": None,
                "requestId": self.requestId,
                "exceptionStack": self.exceptionStack,
            }
        )


class CustomException(BaseException):
    """自定义异常类，继承自 BaseException"""
    pass


def register_exception_handlers(app: FastAPI):
    # 覆盖原生 HTTPException 处理器（处理其他 HTTP 错误）
    @app.exception_handler(HTTPException)
    async def http_exception_handler(request: Request, exc: HTTPException):
        LoggerUtil.error_log(f"Unhandled http_exception_handler: {exc}")
        excFlag = MDC.get("exceptionStack", False)
        exmsg = None
        if excFlag:
            exmsg = format_exception_msg(exc)

        return JSONResponse(
            status_code=exc.status_code,
            content={
                "code": exc.status_code,
                "busiMsg": str(exc.detail),
                "busiCode": exc.status_code,
                "data": None,
                "requestId": TraceID.get_trace(),
                "exceptionStack": exmsg
            },
        )

    # Pydantic 模型校验错误（如直接调用 Pydantic 模型时）
    @app.exception_handler(ValidationError)
    async def pydantic_validation_handler(request: Request, exc: ValidationError):
        error_details = [f"{'.'.join(map(str, e['loc']))}: {e['msg']}" for e in exc.errors()]
        excFlag = MDC.get("exceptionStack", False)
        exmsg = None
        if excFlag:
            exmsg = format_exception_msg(exc)
        return JSONResponse(
            status_code=HttpStatus.BAD_REQUEST,
            content={
                "code": HttpStatus.BAD_REQUEST,
                "busiMsg": f"{ResponseEnum.InvalidRequest.getBusiMsg},{error_details}",
                "busiCode": ResponseEnum.InvalidRequest.getBusiCode,
                "data": None,
                "requestId": TraceID.get_trace(),
                "exceptionStack": exmsg
            }
        )

    @app.exception_handler(RequestValidationError)
    async def validation_exception_handler(request: Request, exc: RequestValidationError):
        LoggerUtil.error_log(f"Unhandled validation_exception_handler: {exc}")
        error_messages = ""
        excFlag = MDC.get("exceptionStack", False)
        exmsg = None
        if excFlag:
            exmsg = format_exception_msg(exc)
        try:
            errors = exc.errors()
            error_messages = []
            for error in errors:
                error_type = error.get("type")
                msg_template = ERROR_TYPE_MAPPING.get(error_type, "{},验证失败")

                msg = ""
                if "{}" in msg_template:
                    msg = msg_template.format(error_type + " " + error.get("msg"),
                                              error.get("ctx", {}).get("limit_value", ""))
                else:
                    msg = error_type + " " + '.'.join(map(str, error['loc'])) + ":" + msg_template

                error_messages.append(msg)
        except Exception as ex:
            LoggerUtil.error_log(f"捕获异常逻辑失败: {ex}")

        return JSONResponse(
            status_code=HttpStatus.BAD_REQUEST,
            content={
                "code": HttpStatus.BAD_REQUEST,
                "busiMsg": "; ".join(error_messages),
                "busiCode": ResponseEnum.InvalidArgument.getBusiCode,
                "data": None,
                "requestId": TraceID.get_trace(),
                "exceptionStack": exmsg
            },
        )

    # 自定义异常处理
    @app.exception_handler(CustomException)
    async def custom_exception_handler(request: Request, exc: CustomException):
        LoggerUtil.error_log(f"Unhandled custom_exception_handler: {exc}")
        res = ResponseEnum.from_code(exc.busiCode)
        status_code = HttpStatus.INTERNAL_SERVER_ERROR
        busiMsg = ResponseEnum.InternalError.getBusiMsg
        busiCode = ResponseEnum.InternalError.getBusiCode
        if res:
            status_code = res.getHttpCode
            busiMsg = res.getBusiMsg
            busiCode = res.getBusiCode
        excFlag = MDC.get("exceptionStack", False)
        exmsg = None
        if excFlag:
            exmsg = format_exception_msg(exc)

        return JSONResponse(
            status_code=status_code,
            content={
                "code": status_code,
                "busiMsg": busiMsg,
                "busiCode": busiCode,
                "data": None,
                "requestId": TraceID.get_trace(),
                "exceptionStack": exmsg
            }
        )

    # 自定义异常处理
    @app.exception_handler(BaseException)
    async def custom_exception_handler(request: Request, exc: BaseException):
        LoggerUtil.error_log(f"Unhandled custom_exception_handler: {exc}")
        excFlag = MDC.get("exceptionStack", False)
        exmsg = None
        if excFlag:
            exmsg = format_exception_msg(exc)
        res = ResponseEnum.from_code(exc.busiCode)
        status_code = HttpStatus.INTERNAL_SERVER_ERROR
        busiMsg = ResponseEnum.InternalError.getBusiMsg
        busiCode = ResponseEnum.InternalError.getBusiCode
        if res:
            status_code = res.getHttpCode
            busiMsg = res.getBusiMsg
            busiCode = res.getBusiCode

        return JSONResponse(
            status_code=status_code,
            content={
                "code": status_code,
                "busiMsg": busiMsg,
                "busiCode": busiCode,
                "data": None,
                "requestId": TraceID.get_trace(),
                "exceptionStack": exmsg
            }
        )

    # 全局异常兜底处理
    @app.exception_handler(Exception)
    async def global_exception_handler(request: Request, exc: Exception):
        # 记录完整的错误堆栈
        LoggerUtil.error_log(f"Unhandled exception: {exc}")
        excFlag = MDC.get("exceptionStack", False)
        exmsg = None
        if excFlag:
            exmsg = format_exception_msg(exc)
        return JSONResponse(
            status_code=ResponseEnum.InternalError.getHttpCode,
            content={
                "code": ResponseEnum.InternalError.getHttpCode,
                "busiMsg": ResponseEnum.InternalError.getBusiMsg,
                "busiCode": ResponseEnum.InternalError.getBusiCode,
                "data": None,
                "requestId": TraceID.get_trace(),
                "exceptionStack": exmsg
            }
        )

    @app.exception_handler(StarletteHTTPException)
    async def http_exception_handler(request: Request, exc: StarletteHTTPException):
        # 记录完整的错误堆栈
        uri = request.url.path
        if Constants.Favicon != uri:
            LoggerUtil.error_log(f"Unhandled exception: {uri},{exc}")
        excFlag = MDC.get("exceptionStack", False)
        exmsg = None
        if excFlag:
            exmsg = format_exception_msg(exc)
        return JSONResponse(
            status_code=ResponseEnum.InvalidRequest.getHttpCode,
            content={
                "code": ResponseEnum.InvalidRequest.getHttpCode,
                "busiMsg": exc.detail,
                "busiCode": ResponseEnum.InvalidRequest.getBusiCode,
                "data": None,
                "requestId": TraceID.get_trace(),
                "exceptionStack": exmsg
            }
        )


def get_project_root_name(file_path: str) -> str:
    """
    通过文件路径回溯获取项目根目录名称
    示例输入: /Users/edy/.../sthg_common_base_0422/sthg_common_base/utils/some_file.py
    返回: sthg_common_base_0422
    """
    # 获取调用文件的绝对路径并提取目录
    current_dir = os.path.dirname(os.path.abspath(file_path))

    # 向上回溯两级目录（根据项目结构调整层级）
    project_root_path = os.path.dirname(os.path.dirname(current_dir))

    # 提取最后一级目录名作为项目名称
    return os.path.basename(project_root_path)


def format_exception_msg(exc: Exception, printEx: bool = True) -> str:
    exmsg = None
    """
    将异常堆栈信息压缩为单行字符串，并聚焦关键错误位置
    :param exc: Exception实例
    :return: 单行格式化的错误信息
    """
    if printEx:
        LoggerUtil.error_log(f"Internal exception: {exc}")

    if not exc:
        return exmsg

    try:
        # 自动获取调用者文件路径
        frame = inspect.currentframe().f_back
        file_path = inspect.getframeinfo(frame).filename

        # 动态获取项目根目录名称
        project_name = get_project_root_name(file_path)

        # 获取完整堆栈
        tb = traceback.format_exception(type(exc), exc, exc.__traceback__)

        # 过滤并格式化关键堆栈
        error_lines = []
        for line in tb:
            line_clean = line.strip()
            if (
                    f"site-packages{os.sep}" not in line_clean  # 过滤第三方库
                    and project_name in line_clean  # 聚焦本项目
            ):
                error_lines.append(line_clean.replace("\n", " "))

        if error_lines and len(tb)>0 and   len(error_lines)<len(tb):
            error_lines.append(tb[-1])
        # 提取核心错误（最后一行）并合并为单行
        if error_lines and len(error_lines) > 0:
            exmsg = ",\n    ".join(error_lines)

    except Exception as e:
        return LoggerUtil.error_log(f"{e}")
    return exmsg



from build.lib.sthg_common_base import LoggerUtil
from sthg_common_base.response.httpCodeEnum import ResponseEnum
from sthg_common_base.response.response import BaseResponse
from typing import Optional, Any, Union
from datetime import datetime


class ValidCommon:
    """
    字段验证工具类，提供各种字段验证方法
    所有方法返回BaseResponse对象:
    - 验证成功: ResponseEnum.OK
    - 验证失败: ResponseEnum.InvalidArgument
    - 异常错误: ResponseEnum.InternalError
    """

    @staticmethod
    def check_null(
            value: Any,
            field_name: str = "字段",
            base_response: BaseResponse = None
    ) -> BaseResponse:
        """
        检查字段是否为None或空字符串
        :param base_response: 如果已有失败的响应，则直接返回
        """
        # 如果已有失败的响应，直接返回
        if base_response and not base_response.is_success():
            return base_response

        try:
            if value is None:
                resp = BaseResponse(ResponseEnum.InvalidArgument, msg=f"{field_name}不能为空")
                if base_response:
                    base_response.build_reset_by_resp(resp)
                    return base_response
                return resp

            if isinstance(value, str) and value.strip() == "":
                resp = BaseResponse(ResponseEnum.InvalidArgument, msg=f"{field_name}不能为空字符串")
                if base_response:
                    base_response.build_reset_by_resp(resp)
                    return base_response
                return resp

            return BaseResponse(ResponseEnum.OK, data=value)
        except Exception as e:
            LoggerUtil.error_log(f"{field_name}:检查空值异常")
            resp = BaseResponse(
                ResponseEnum.InternalError,
                msg=f"{field_name}:检查空值异常"
            )
            if base_response:
                base_response.build_reset_by_resp(resp)
                return base_response
            return resp

    @staticmethod
    def check_numeric(
            value: Union[str, int, float],
            field_name: str = "字段",
            base_response: BaseResponse = None
    ) -> BaseResponse:
        """
        检查字段是否为数字或可转换为数字
        :param base_response: 如果已有失败的响应，则直接返回
        """
        # 如果已有失败的响应，直接返回
        if base_response and not base_response.is_success():
            return base_response

        try:
            # 如果已经是数字类型
            if isinstance(value, (int, float)):
                return BaseResponse(ResponseEnum.OK, data=value)

            # 尝试转换字符串为数字
            if isinstance(value, str):
                if value.isdigit():
                    return BaseResponse(ResponseEnum.OK, data=int(value))
                try:
                    float_val = float(value)
                    return BaseResponse(ResponseEnum.OK, data=float_val)
                except ValueError:
                    pass

            resp = BaseResponse(ResponseEnum.InvalidArgument, msg=f"{field_name}必须是数字")
            if base_response:
                base_response.build_reset_by_resp(resp)
                return base_response
            return resp
        except Exception as e:
            LoggerUtil.error_log(f"{field_name}:数字检查异常")
            resp = BaseResponse(
                ResponseEnum.InternalError,
                msg=f"{field_name}:数字检查异常"
            )
            if base_response:
                base_response.build_reset_by_resp(resp)
                return base_response
            return resp

    @staticmethod
    def validate_length(
            value: str,
            min_len: Optional[int] = None,
            max_len: Optional[int] = None,
            field_name: str = "字段",
            base_response: BaseResponse = None
    ) -> BaseResponse:
        """
        验证字符串长度范围
        :param base_response: 如果已有失败的响应，则直接返回
        """
        # 如果已有失败的响应，直接返回
        if base_response and not base_response.is_success():
            return base_response

        try:
            # 先检查是否为字符串
            if not isinstance(value, str):
                resp = BaseResponse(ResponseEnum.InvalidArgument, msg=f"{field_name}必须是字符串类型")
                if base_response:
                    base_response.build_reset_by_resp(resp)
                    return base_response
                return resp

            length = len(value)
            error_msg = None

            if min_len is not None and max_len is not None:
                if not (min_len <= length <= max_len):
                    error_msg = f"{field_name}长度需在{min_len}到{max_len}之间"
            elif min_len is not None:
                if length < min_len:
                    error_msg = f"{field_name}长度不能小于{min_len}"
            elif max_len is not None:
                if length > max_len:
                    error_msg = f"{field_name}长度不能超过{max_len}"

            if error_msg:
                resp = BaseResponse(ResponseEnum.InvalidArgument, msg=error_msg)
                if base_response:
                    base_response.build_reset_by_resp(resp)
                    return base_response
                return resp

            return BaseResponse(ResponseEnum.OK, data=value)
        except Exception as e:
            LoggerUtil.error_log(f"{field_name}:长度验证异常")
            resp = BaseResponse(
                ResponseEnum.InternalError,
                msg=f"{field_name}:长度验证异常"
            )
            if base_response:
                base_response.build_reset_by_resp(resp)
                return base_response
            return resp

    @staticmethod
    def validate_date_format(
            value: str,
            date_format: str = "%Y-%m-%d",
            field_name: str = "字段",
            base_response: BaseResponse = None
    ) -> BaseResponse:
        """
        验证日期格式是否符合指定格式
        :param base_response: 如果已有失败的响应，则直接返回
        """
        # 如果已有失败的响应，直接返回
        if base_response and not base_response.is_success():
            return base_response

        try:
            if not isinstance(value, str):
                resp = BaseResponse(ResponseEnum.InvalidArgument, msg=f"{field_name}必须是字符串类型")
                if base_response:
                    base_response.build_reset_by_resp(resp)
                    return base_response
                return resp

            try:
                # 尝试解析日期
                datetime.strptime(value, date_format)
                return BaseResponse(ResponseEnum.OK, data=value)
            except ValueError:
                resp = BaseResponse(
                    ResponseEnum.InvalidArgument,
                    msg=f"{field_name}格式应为{date_format}"
                )
                if base_response:
                    base_response.build_reset_by_resp(resp)
                    return base_response
                return resp
        except Exception as e:
            LoggerUtil.error_log(f"{field_name}:日期格式验证异常")
            resp = BaseResponse(
                ResponseEnum.InternalError,
                msg=f"{field_name}:日期格式验证异常"
            )
            if base_response:
                base_response.build_reset_by_resp(resp)
                return base_response
            return resp

# 使用示例
if __name__ == "__main__":
    # 测试空值检查
    print("空值检查测试:")
    print(ValidCommon.check_null(None).to_dict())  # 应失败
    print(ValidCommon.check_null("").to_dict())  # 应失败
    print(ValidCommon.check_null("valid").to_dict())  # 应成功

    # 测试数字检查
    print("\n数字检查测试:")
    print(ValidCommon.check_numeric("123").to_dict())  # 应成功 (整数)
    print(ValidCommon.check_numeric("123.45").to_dict())  # 应成功 (浮点数)
    print(ValidCommon.check_numeric("abc").to_dict())  # 应失败

    # 测试长度验证
    print("\n长度验证测试:")
    print(ValidCommon.validate_length("test", min_len=3, max_len=5).to_dict())  # 应成功
    print(ValidCommon.validate_length("te", min_len=3).to_dict())  # 应失败
    print(ValidCommon.validate_length("too_long", max_len=5).to_dict())  # 应失败

    # 测试日期格式
    print("\n日期格式测试:")
    print(ValidCommon.validate_date_format("2023-08-15").to_dict())  # 应成功
    print(ValidCommon.validate_date_format("15/08/2023", date_format="%d/%m/%Y").to_dict())  # 应成功
    print(ValidCommon.validate_date_format("2023/08/15").to_dict())  # 应失败 (格式不匹配)


from .response.exception import BaseException,CustomException,register_exception_handlers
from .response.httpCodeEnum import HttpStatus, ResponseEnum, ResCode
from .response.response import BaseResponse
from .utils.log_util import LoggerUtil,MDC,TraceLogger,register_log_middleware
from .utils.log_wrapper import access_log,service_log,register_exception
from .response.exception import format_exception_msg
from .response.json_serializer import EnhancedJSONSerializer




from enum import Enum


class SthgCommonConstants:
    header_x_user_id_key = "x-user-id"
    header_user_id_key = "user_id"
    header_x_request_id_key = "x-request-id"
    header_x_trace_id_key = "x-trace-id"
    header_x_guid_id_key = "x-guid"
    context_trace_id_key = "x_trace_id"
    context_request_id_key = "_x_request_id"
    context_user_id_key = "user_id"
    context_guid_id_key = "x_guid"


class SthgResourceType(Enum):
    # Ontology 类型
    OBJECT_TYPE = "object-type"
    LINK_TYPE = "link-type"
    ACTION_TYPE = "action-type"
    # Function Registry
    CODE_FUNCTION = "function"
    # Foundry
    CODE_PROJECT = "project"
    BRANCH = "branch"
    WORKFLOW = "workflow"
    FLYFLOW = "flyflow"
    PIPELINE = "pipeline"
    DATASET = "dataset"
    # Compass
    DATAEASE = "dataease"
    # 文件夹
    FOLDER = "folder"
    SPACE="space"
    PROJECT="project"


"""
@Author  ：duomei
@File    ：generator.py
@Time    ：2025/9/8 16:46
"""
import uuid
from enum import Enum

from sthg_common_base.utils.sthg_common_constants import SthgResourceType


# ===== 定义 ResourcePool 枚举，每个值是 (scope, resource_type 枚举列表) =====
class ResourcePool(Enum):
    ONTOLOGY = ("ontology", [SthgResourceType.OBJECT_TYPE, SthgResourceType.LINK_TYPE, SthgResourceType.ACTION_TYPE])
    FUNCTION = ("function-registry", [SthgResourceType.CODE_FUNCTION])
    FOUNDRY = ("foundry", [
        SthgResourceType.CODE_PROJECT, SthgResourceType.BRANCH, SthgResourceType.WORKFLOW,
        SthgResourceType.FLYFLOW, SthgResourceType.PIPELINE, SthgResourceType.DATASET
    ])
    COMPASS = ("compass", [SthgResourceType.DATAEASE, SthgResourceType.FOLDER])

class GeneratorBase:
    # ===== 通用 RID 生成函数 =====
    def generate_rid(
        self,
        resource_type: str,  # 支持SthgResourceType和str类型
        namespace: str = None,
        scope: str = None  # 新增参数，允许外部传入 scope
    ) -> str:
        """
        根据资源类型生成 RID，兼容SthgResourceType枚举和字符串

        :param resource_type: 资源类型，可以是SthgResourceType枚举或对应的字符串值
        :param namespace: 可选命名空间，默认 main
        :param scope: 可选 scope，如果传入则使用，否则自动匹配 ResourcePool
        :return: ri.xxx.main.xxx.xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        """
        # 处理资源类型，将字符串转换为对应的枚举
        if isinstance(resource_type, str):
            try:
                resource_type = SthgResourceType(resource_type)
                resource_type = resource_type.value
            except ValueError:
                if scope is None:
                    raise ValueError(f"自定义类型必须指定scope")
        elif not isinstance(resource_type, SthgResourceType):
            raise TypeError("resource_type 必须是 SthgResourceType 枚举或字符串")

        found_scope = scope
        if not found_scope:
            for pool in ResourcePool:
                pool_scope, types = pool.value
                if resource_type in [t.value for t in types]:
                    found_scope = pool_scope
                    break

        if not found_scope:
            valid_types = [t.value for pool in ResourcePool for t in pool.value[1]]
            raise ValueError(
                f"未知 resource_type '{resource_type}'，可选类型: {valid_types}"
            )
        if not namespace:
            namespace = "main"
        rid_uuid = str(uuid.uuid4())
        return f"ri.{found_scope.lower()}.{namespace.lower()}.{resource_type.lower()}.{rid_uuid}"



"""
@Author  ：duomei
@File    ：foundry_type.py
@Time    ：2025/9/12 11:32
"""
from enum import Enum


class FoundryResourceType(Enum):
    """资源枚举类，同时包含英文标识和中文翻译"""
    AIP_PROFILE = ("AIP_PROFILE", "AIP 配置文件")
    AIP_ASSIST_WALKTHROUGH = ("AIP_ASSIST_WALKTHROUGH", "AIP 辅助引导")
    ARTIFACTS_REPOSITORY = ("ARTIFACTS_REPOSITORY", "构件仓库")
    BELLASO_CIPHER_CHANNEL = ("BELLASO_CIPHER_CHANNEL", "贝拉索密码频道")
    BELLASO_CIPHER_LICENSE = ("BELLASO_CIPHER_LICENSE", "贝拉索密码许可证")
    BLACKSMITH_DOCUMENT = ("BLACKSMITH_DOCUMENT", "铁匠文档")
    BLOBSTER_ARCHIVE = ("BLOBSTER_ARCHIVE", "Blobster 归档文件")
    BLOBSTER_AUDIO = ("BLOBSTER_AUDIO", "Blobster 音频文件")
    BLOBSTER_BLOB = ("BLOBSTER_BLOB", "Blobster 二进制大对象")
    BLOBSTER_CODE = ("BLOBSTER_CODE", "Blobster 代码文件")
    BLOBSTER_CONFIGURATION = ("BLOBSTER_CONFIGURATION", "Blobster 配置文件")
    BLOBSTER_DOCUMENT = ("BLOBSTER_DOCUMENT", "Blobster 文档")
    BLOBSTER_IMAGE = ("BLOBSTER_IMAGE", "Blobster 图像文件")
    BLOBSTER_JUPYTERNOTEBOOK = ("BLOBSTER_JUPYTERNOTEBOOK", "Blobster Jupyter 笔记本")
    BLOBSTER_PDF = ("BLOBSTER_PDF", "Blobster PDF 文件")
    BLOBSTER_PRESENTATION = ("BLOBSTER_PRESENTATION", "Blobster 演示文稿")
    BLOBSTER_SPREADSHEET = ("BLOBSTER_SPREADSHEET", "Blobster 电子表格")
    BLOBSTER_VIDEO = ("BLOBSTER_VIDEO", "Blobster 视频文件")
    BLOBSTER_XML = ("BLOBSTER_XML", "Blobster XML 文件")
    CARBON_WORKSPACE = ("CARBON_WORKSPACE", "Carbon 工作空间")
    COMPASS_FOLDER = ("COMPASS_FOLDER", "Compass 文件夹")
    COMPASS_WEB_LINK = ("COMPASS_WEB_LINK", "Compass 网页链接")
    CONTOUR_ANALYSIS = ("CONTOUR_ANALYSIS", "轮廓分析")
    DATA_HEALTH_MONITORING_VIEW = ("DATA_HEALTH_MONITORING_VIEW", "数据健康监控视图")
    DECISIONS_EXPLORATION = ("DECISIONS_EXPLORATION", "决策探索")
    DREDDIE_PIPELINE = ("DREDDIE_PIPELINE", "Dreddie 流水线")
    EDDIE_LOGIC = ("EDDIE_LOGIC", "Eddie 逻辑")
    EDDIE_PIPELINE = ("EDDIE_PIPELINE", "Eddie 流水线")
    FFORMS_FORM = ("FFORMS_FORM", "FForms 表单")
    FLOW_WORKFLOW = ("FLOW_WORKFLOW", "Flow 工作流")
    FOUNDRY_DATASET = ("FOUNDRY_DATASET", "Foundry 数据集")
    FOUNDRY_DEPLOYED_APP = ("FOUNDRY_DEPLOYED_APP", "Foundry 已部署应用")
    FOUNDRY_ACADEMY_TUTORIAL = ("FOUNDRY_ACADEMY_TUTORIAL", "Foundry 学院教程")
    FOUNDRY_CONTAINER_SERVICE_CONTAINER = ("FOUNDRY_CONTAINER_SERVICE_CONTAINER", "Foundry 容器服务容器")
    FOUNDRY_ML_OBJECTIVE = ("FOUNDRY_ML_OBJECTIVE", "Foundry 机器学习目标")
    FOUNDRY_TEMPLATES_TEMPLATE = ("FOUNDRY_TEMPLATES_TEMPLATE", "Foundry 模板（模板文件）")
    FUSION_DOCUMENT = ("FUSION_DOCUMENT", "Fusion 文档")
    GPS_VIEW = ("GPS_VIEW", "GPS 视图")
    HUBBLE_EXPLORATION_LAYOUT = ("HUBBLE_EXPLORATION_LAYOUT", "Hubble 探索布局")
    HYPERAUTO_INTEGRATION = ("HYPERAUTO_INTEGRATION", "HyperAuto 集成")
    LOGIC_FLOWS_CONNECTED_FLOW = ("LOGIC_FLOWS_CONNECTED_FLOW", "Logic Flows 关联流")
    MACHINERY_DOCUMENT = ("MACHINERY_DOCUMENT", "机械文档")
    MAGRITTE_AGENT = ("MAGRITTE_AGENT", "马格里特代理")
    MAGRITTE_DRIVER = ("MAGRITTE_DRIVER", "马格里特驱动程序")
    MAGRITTE_SOURCE = ("MAGRITTE_SOURCE", "马格里特数据源")
    MARKETPLACE_BLOCK_SET_INSTALLATION = ("MARKETPLACE_BLOCK_SET_INSTALLATION", "Marketplace 块集安装")
    MARKETPLACE_BLOCK_SET_REPO = ("MARKETPLACE_BLOCK_SET_REPO", "Marketplace 块集仓库")
    MARKETPLACE_LOCAL = ("MARKETPLACE_LOCAL", "本地 Marketplace")
    MARKETPLACE_REMOTE_STORE = ("MARKETPLACE_REMOTE_STORE", "Marketplace 远程存储")
    MIO_MEDIA_SET = ("MIO_MEDIA_SET", "MIO 媒体集")
    MODELS_MODEL = ("MODELS_MODEL", "模型（模型文件）")
    MODELS_MODEL_VERSION = ("MODELS_MODEL_VERSION", "模型版本")
    MONOCLE_GRAPH = ("MONOCLE_GRAPH", "Monocle 图表")
    NOTEPAD_NOTEPAD = ("NOTEPAD_NOTEPAD", "记事本（记事本文件）")
    NOTEPAD_NOTEPAD_TEMPLATE = ("NOTEPAD_NOTEPAD_TEMPLATE", "记事本模板")
    OBJECT_SENTINEL_MONITOR = ("OBJECT_SENTINEL_MONITOR", "对象哨兵监控")
    OBJECT_SET_VERSIONED_OBJECT_SET = ("OBJECT_SET_VERSIONED_OBJECT_SET", "对象集（带版本的对象集）")
    OPUS_GRAPH = ("OPUS_GRAPH", "Opus 图表")
    OPUS_GRAPH_TEMPLATE = ("OPUS_GRAPH_TEMPLATE", "Opus 图表模板")
    OPUS_MAP = ("OPUS_MAP", "Opus 地图")
    OPUS_MAP_LAYER = ("OPUS_MAP_LAYER", "Opus 地图图层")
    OPUS_MAP_TEMPLATE = ("OPUS_MAP_TEMPLATE", "Opus 地图模板")
    OPUS_SEARCH_AROUND = ("OPUS_SEARCH_AROUND", "Opus 周边搜索")
    QUIVER_ANALYSIS = ("QUIVER_ANALYSIS", "Quiver 分析")
    QUIVER_ARTIFACT = ("QUIVER_ARTIFACT", "Quiver 构件")
    QUIVER_DASHBOARD = ("QUIVER_DASHBOARD", "Quiver 仪表盘")
    REPORT_REPORT = ("REPORT_REPORT", "报告（报告文件）")
    SLATE_DOCUMENT = ("SLATE_DOCUMENT", "Slate 文档")
    SOLUTION_DESIGN_DIAGRAM = ("SOLUTION_DESIGN_DIAGRAM", "解决方案设计图")
    STEMMA_REPOSITORY = ("STEMMA_REPOSITORY", "Stemma 仓库")
    TABLES_TABLE = ("TABLES_TABLE", "表格（表格文件）")
    TAURUS_WORKFLOW = ("TAURUS_WORKFLOW", "Taurus 工作流")
    THIRD_PARTY_APPLICATIONS_APPLICATION = ("THIRD_PARTY_APPLICATIONS_APPLICATION", "第三方应用（应用程序）")
    TIME_SERIES_CATALOG_SYNC = ("TIME_SERIES_CATALOG_SYNC", "时间序列目录同步")
    VECTOR_TEMPLATE = ("VECTOR_TEMPLATE", "Vector 模板")
    VECTOR_WORKBOOK = ("VECTOR_WORKBOOK", "Vector 工作簿")
    WORKSHOP_MODULE = ("WORKSHOP_MODULE", "Workshop 模块")
    WORKSHOP_STATE = ("WORKSHOP_STATE", "Workshop 状态")
    CODE_REPOSITORY_FUNCTION = ("CODE_REPOSITORY_FUNCTION", "代码仓库函数")
    COMPASS_DATAEASE = ("COMPASS_DATAEASE", "dataEase")
    FOUNDRY_FLYFLOW = ("foundry_FLYFLOW", "审批流")

    @property
    def english(self):
        """返回英文标识"""
        return self.value[0]

    @property
    def chinese(self):
        """返回中文翻译"""
        return self.value[1]



import os
import inspect
import sys
import configparser
from pathlib import Path
from typing import Optional

def _get_root_project_name() -> Optional[str]:
    """安全回溯获取项目根目录名称（静默模式）"""

    def _find_root(start: Path) -> Optional[Path]:
        """静默查找项目根目录"""
        if not start.exists():
            return None

        for parent in start.parents:
            # 检测常见项目标识文件
            if any((parent / marker).exists() for marker in [
                '.git',
                'pyproject.toml',
                'setup.py',
                'requirements.txt'
            ]):
                return parent
            if parent == parent.parent:  # 到达系统根目录
                break
        return None

    # 获取入口路径（多级降级）
    entry_path = None
    try:
        # 方法1：调用栈分析
        frame = inspect.currentframe()
        while frame and frame.f_back:
            frame = frame.f_back
        if module := inspect.getmodule(frame):
            if hasattr(module, '__file__') and module.__file__:
                entry_path = Path(module.__file__).resolve()
    except Exception:
        pass

    # 方法2：命令行参数
    if not entry_path and sys.argv[0]:
        try:
            entry_path = Path(sys.argv[0]).resolve()
        except Exception:
            pass

    # 方法3：当前工作目录
    if not entry_path:
        entry_path = Path.cwd()

    # 执行查找
    if root := _find_root(entry_path):
        return root.name
    return None

def _get_config_project_name() -> Optional[str]:
    """静默模式读取配置（支持 YAML）"""
    config_files = [
        Path("pyproject.toml"),
        Path("setup.cfg"),
        Path("settings.ini"),
        Path("config/settings.ini"),
        Path("config/settings.yaml"),
        Path("settings.yaml"),
        Path("config/settings.yml"),
        Path("settings.yml")
    ]

    for cfg_path in config_files:
        if not cfg_path.exists():
            continue

        try:
            # YAML 格式处理
            if cfg_path.suffix in ('.yaml', '.yml'):
                try:
                    import yaml  # 需要 PyYAML 库
                    with open(cfg_path, 'r') as f:
                        data = yaml.safe_load(f)
                        # 多层级尝试获取名称
                        return (
                            data.get('project', {}).get('name')  # project.name
                            or data.get('name')  # 顶层 name
                            or data.get('metadata', {}).get('name')  # metadata.name
                        )
                except ImportError:  # 未安装 PyYAML
                    continue
                except Exception:  # 解析失败
                    continue

            # TOML 格式处理
            if cfg_path.name == 'pyproject.toml':
                try:
                    import toml
                    data = toml.load(cfg_path)
                    return data.get('project', {}).get('name') or \
                           data.get('tool', {}).get('poetry', {}).get('name')
                except ImportError:
                    pass
                except Exception:
                    continue

            # INI 格式处理
            config = configparser.ConfigParser()
            config.read(cfg_path)
            if config.has_section('project'):
                return config.get('project', 'name', fallback=None)
            if config.has_section('metadata'):
                return config.get('metadata', 'name', fallback=None)

        except Exception:
            continue

    return None

def get_project_name(default: str = "unknown_project") -> str:
    """
    安全获取项目名称（三级降级策略）
    Args:
        default: 所有方法失败时的默认返回值
    Returns:
        项目名称或默认值（永远不会抛出异常）
    """
    # 第一级：配置读取

    try:
        if name := os.getenv("PROJECT_NAME",None):
            return name
        if name := _get_config_project_name():
            return name
        # 第二级：路径回溯
        if name := _get_root_project_name():
            return name
        # 第三级：当前目录名称
        if name := Path.cwd().name:
            return name
    except Exception:
        return default

    return default
# 示例用法
if __name__ == '__main__':
    print(f"Detected project name: {get_project_name()}")